package jp.sourceforge.jirc.net;

import java.io.IOException;
import java.text.DateFormat;
import java.util.*;

import jp.sourceforge.jirc.text.IRCMessageFormat;

class CommandMap {
    private String CTCP_VERSION = "jIRC version ALPHA";
    private IRC irc;
    private Hashtable commandList = new Hashtable();

    public CommandMap(IRC irc) {
    		this.irc = irc;
    		commandList.put("KILL", new KILL());
    		commandList.put("PING", new PING());
    		commandList.put("NICK", new NICK());
    		commandList.put("JOIN", new JOIN());
    		commandList.put("PART", new PART());
    		commandList.put("MODE", new MODE());
    		commandList.put("QUIT", new QUIT());
    		commandList.put("KICK", new KICK());
    		commandList.put("ERROR", new ERROR());
    		commandList.put("TOPIC", new TOPIC());
    		commandList.put("NOTICE", new NOTICE());
    		commandList.put("INVITE", new INVITE());
    		commandList.put("PRIVMSG", new PRIVMSG());
    		commandList.put("WALLOPS", new WALLOPS());
    		commandList.put("NUMERICREPLY", new NUMERICREPLY(irc));
    }

    public Command getCommand(String commandName) {
    		return (Command )commandList.get(commandName);
    }

    Channel getChannelOrPriv(String channelname, String nick) {
    		Channel channel = irc.getChannel(channelname);
    		if(channel == null) {
    			channel = irc.getChannel(nick);
    			if(channel == null) {
    				channel = irc.makeChannel(nick);
    			}
    		}

    		return channel;
    }

    static String removeColon(String string) {
    		if(string.indexOf(':') != 0) {
    			return string;
    		} else {
    			return string.substring(string.indexOf(':') +1);
    		}
    }

    static String getNick(String prefix){
    		int i = prefix.indexOf('!');
    		if(i > 0) {
    			prefix = removeColon(prefix).substring(0, i - 1);
    		} else {
    			prefix = removeColon(prefix);
    		}
	
    		return prefix;
    }

    private abstract class AbstractCommand implements Command {
    		public void execute(String prefix, String params) {};
    		public void execute(String params) {};
    }

    private class QUIT extends AbstractCommand {
    		public void execute(String prefix, String params) {
    			String nick = getNick(prefix);
    			if(nick.equalsIgnoreCase(irc.getNick())) {
    				irc.close();
    			} else {
    				irc.removeNick(nick, removeColon(params));
    			}
    		}
    }

    private class INVITE extends AbstractCommand {
    		public void execute(String prefix, String params) {
    			String nick = getNick(prefix);
	    
    			StringTokenizer st = new StringTokenizer(params, " ",false);
    			String target = st.nextToken(); // channel or yournick
    			String channel = removeColon(st.nextToken(CRLF).trim());	    

    			String[] args = {nick, target, channel};
    			irc.getChannel(irc.getServer()).appendCommandMessage(IRCMessageFormat.getInviteMessage(args));
    		}
    }

    private class WALLOPS extends AbstractCommand {
    		public void execute(String prefix, String params) {
    			String[] args = {prefix, removeColon(params)};
    			irc.getChannel(irc.getServer()).appendCommandMessage(IRCMessageFormat.getWallopsMessage(args));
    		}
    }

    private class KILL extends AbstractCommand {
    		public void execute(String prefix, String params) {
    			String nick = getNick(prefix);
	    
    			StringTokenizer st = new StringTokenizer(params, " ",false);
    			String channelName = st.nextToken(); // channel or yournick
    			String message = removeColon(st.nextToken(CRLF).trim());	    

    			String[] args = {nick, channelName, message};
    			irc.getChannel(irc.getServer()).appendCommandMessage(IRCMessageFormat.getKillMessage(args));
    			irc.close();
    		}
    }
    
    private class ERROR extends AbstractCommand {
    		public void execute(String params) {
    			String errorMessage = removeColon(params);
    			Enumeration channels = irc.getChannels();

    			while(channels.hasMoreElements()) {
    				Channel channel = (Channel )channels.nextElement();
    				channel.appendErrorMessage(errorMessage);
    			}
    			irc.close();
    		}
    }

    private class PING extends AbstractCommand {
    		public void execute(String params) {
    			try {
    				irc.PONG(params);
    			} catch(IOException e) {
    				irc.close();
    			}
    		}
    }

    private class NICK extends AbstractCommand {
    		public void execute(String prefix, String params) {
    			String oldNick = getNick(prefix);
    			String newNick = removeColon(params);
	    
    			irc.changeNick(oldNick, newNick);
    		}
    }

    private class JOIN extends AbstractCommand {
    		public void execute(String prefix, String params) {
    			String nick = getNick(prefix);
    			String channelName = removeColon(params);
	    
    			if(nick.equals(irc.getNick())) {
    				irc.makeChannel(channelName);
    				try {
    					irc.protectedMODE(channelName);
    				} catch(IOException e) {
    					e.printStackTrace();
    				}
    			} else {
    				Channel channel = irc.getChannel(channelName);
		
    				String[] args = { nick, channelName };
    				channel.appendCommandMessage(IRCMessageFormat.getJoinMessage(args));
    				channel.addUser(nick);
    			}
    		}
    }

    private class PART extends AbstractCommand {
    		public void execute(String prefix, String params) {
    			String nick = getNick(prefix);
	    
    			StringTokenizer st = new StringTokenizer(params, " ",false);
    			String channelName = st.nextToken(); // channel or yournick
    			String message = removeColon(st.nextToken(CRLF).trim());	    
    			Channel channel = irc.getChannel(channelName);

    			if(nick.equalsIgnoreCase(irc.getNick())) {
    				irc.removeChannel(channel);
    				channel = irc.getChannel(irc.getServer());
    			} else {
    				channel.removeUser(nick);
    			}

    			String[] args = {nick, channelName, message};
    			channel.appendCommandMessage(IRCMessageFormat.getPartMessage(args));
    		}
    }

    private class TOPIC extends AbstractCommand {
    		public void execute(String prefix, String params) {
    			StringTokenizer st = new StringTokenizer(params, " ",false);

    			String channelname = st.nextToken(); // channel or yournick
    			String topic = removeColon(st.nextToken(CRLF).trim());
	    
    			Channel channel = irc.getChannel(channelname);
	    
    			channel.setTopic(topic);
    			String[] args = {getNick(prefix), channelname, topic};
    			channel.appendCommandMessage(IRCMessageFormat.getTopicMessage(args));
    		}
    }

    private class MODE extends AbstractCommand {
    		public void execute(String prefix, String params) {
    			StringTokenizer st = new StringTokenizer(params, " ", false);
    			String channelname = st.nextToken(); // channel or yournick
    			String mode = st.nextToken(CRLF).trim();
	    
    			Channel channel;
    			if(channelname.equalsIgnoreCase(irc.getNick())) {
    				String[] args = {channelname, mode};
    				String message = IRCMessageFormat.getModeMessage(args);
    				channel = irc.getChannel(irc.getServer());
    				channel.appendCommandMessage(message);
    				irc.getClient().append(channel, message);
    			} else if((channel = irc.getChannel(channelname)) != null) {
    				channel.setMode(getNick(prefix), mode);
    			}
    		}
    }
    
    private class KICK extends AbstractCommand {
    		public void execute(String prefix, String params) {
    			String nick = getNick(prefix);
	    
    			StringTokenizer st = new StringTokenizer(params, " ",false);
    			String channelName = st.nextToken();
    			String target = st.nextToken();
    			String message = removeColon(st.nextToken(CRLF).trim());	    
    			Channel channel = irc.getChannel(channelName);

    			if(target.equalsIgnoreCase(irc.getNick())) {
    				irc.removeChannel(channel);
    				channel = irc.getChannel(irc.getServer());
    			} else {
    				channel.removeUser(target);
    			}

    			String[] args = {nick, channelName, target, message};
    			channel.appendCommandMessage(IRCMessageFormat.getKickMessage(args));
    		}
    }

    private class NOTICE extends AbstractCommand {
    		public void execute(String prefix, String params) {
    			String nick = getNick(prefix);
	    
    			StringTokenizer st = new StringTokenizer(params, " ",false);
    			String channelname = st.nextToken(); // channel or yournick
    			String message = removeColon(st.nextToken(CRLF).trim());
	    
    			Channel channel = getChannelOrPriv(channelname, nick);

    			String[] args = {nick, channelname, message};
    			channel.append(IRCMessageFormat.getNoticeMessage(args));
    			irc.getClient().append(channel, IRCMessageFormat.getNoticeFullMessage(args));
    		}
    }

    public class PRIVMSG extends AbstractCommand {
    		public void execute(String prefix, String params) {
    			String nick = getNick(prefix);
	    
    			StringTokenizer st = new StringTokenizer(params, " ",false);
    			String channelname = st.nextToken(); // channel or yournick
    			String message = removeColon(st.nextToken(CRLF).trim());
	    
    			// if message.length < 0, StringIndexOutOfBoundsException
    			if((message.length() > 0) && (message.charAt(0) == '\u0001')) {
    				int j = 0;
    				for(int i = 2; i < message.length(); i++) {
    					if(message.charAt(i) == '\u0001') {
    						j = i;
    					}
    				}
    				if(j ==0) {
    					j = message.length();
    				}
    				ctcp(nick, channelname, message.substring(1, j));
    			} else {
    				Channel channel = getChannelOrPriv(channelname, nick);

    				String[] args = {nick, channelname, message};
    				channel.append(IRCMessageFormat.getPrivmsgMessage(args));
    				irc.getClient().append(channel, IRCMessageFormat.getPrivmsgFullMessage(args));
    			}
    		}
	
    		private void ctcp(String nick, String channelname, String value) {
    			try {
        			String ctcp = null;
        			String replay = null;
    				if(value.startsWith("PING")) {
    					ctcp   = "PING";
    					replay = value;
    				} else if(value.startsWith("VERSION")) {
    					ctcp   = "VERSION";
    					replay = CTCP_VERSION;
    				} else if(value.startsWith("TIME")) {
    					ctcp   = "TIME";
    					DateFormat date = DateFormat.getDateTimeInstance(DateFormat.LONG, DateFormat.LONG);
    					replay = "TIME " + date.format(new Date());
    				} else if(value.startsWith("CLIENTINFO")) {
    					replay = "CLIENTINFO PING TIME VERSION";
    			//  } else if(value.startsWith("USERINFO")) {
		    //		replay = "USERINFO";
    				} else if(value.startsWith("ACTION ")) {
    					Channel channel = irc.getChannel(channelname);
        				String[] args = {nick, channelname, value.substring(7)};
        				channel.append(IRCMessageFormat.getActionMessage(args));
//      				irc.getClient().append(channel, IRCMessageFormat.getActionMessage(args));
    				}
    				
    				if(ctcp != null) {
        				irc.NOTICE(nick, '\u0001' + replay + '\u0001');

        				String[] args = {nick, ctcp};
    					irc.getClient().append(irc.getChannel(irc.getServer()), IRCMessageFormat.getCTCPMessage(args));
    				}
    			} catch(Exception e) {
    				e.printStackTrace();
    			}
    		}
    }
}
