package jp.sourceforge.jirc.net;

import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.StringTokenizer;

public final class IRC {
	private boolean DEBUG = false;
    private boolean connected = false;

    private String nick = null;
    private String user = null;
    private String pass = null;
    private String server = null;
    
    private IRCIO io;
    private Client client = null;
    private CommandMap commandMap = new CommandMap(this);

    private Date date;
    private final Hashtable channelList = new Hashtable();
    private final String CRLF = "\r\n";

    public IRC(String server, int port, String nick, String user, String pass) throws Exception {
    		String localhost = null;
	
    		try {
    			localhost = InetAddress.getLocalHost().getHostName();
    		} catch (UnknownHostException e) {
    			localhost = "unknown";

    			throw e;
    		}
	
    		this.nick = nick;
    		this.user = new String(nick+" "+localhost+" "+ server + " :" + user);
    		this.pass = pass;
    		this.server = server;
	
    		try {
    			date = new Date();
    			io = new IRCIO(this, server, port);
    		} catch(Exception e) {
    			io = null;
    			date = null;

    			e.printStackTrace();

    			throw e;
    		}

    		/*
    		 System.out.println("#IRC INIT");
    		 Thread.currentThread().getThreadGroup().list();
    		 */
    	}

    public boolean isConnected() {
    		return connected;
    }

    protected synchronized void setConnected(boolean flag) {
    		if(flag) {
    			notify();
    		}
    		this.connected = flag;
    		client.setConnection(this, flag);
    }
    
    public final void start() throws IOException {
    		addChannel(client.makeServerChannel(this, server));
    		io.start();
    		if(pass.length() > 0) {
    			PASS(pass);
    		}
    		USER(user);
    		NICK(nick);

    		/*
    		System.out.println("#IRC START");
    		Thread.currentThread().getThreadGroup().list();
    		*/
    }

    public final void close() {
    		try {
    			setConnected(false);
    			io.close();
    			io.interrupt();
    			io.join();
    		} catch(InterruptedException e) {
    			setException(e);
    		} catch(Exception e) {
    			setException(e);
    		}

    		/*
    		 System.out.println("#IRC CLOSE");
    		 Thread.currentThread().getThreadGroup().list();
    		 */
    }
    
    public String getNick() {
    		return nick;
    }
    
    protected void setNick(String nick) {
    		this.nick = nick;
    }

    protected void changeNick(String oldNick, String newNick) {
    		if(oldNick.equalsIgnoreCase(nick)) {
    			setNick(newNick);
    		}

    		client.changeUser(this, oldNick, newNick);
    }

    protected void removeNick(String nick, String message) {
    		if(!nick.equalsIgnoreCase(this.nick)) {
    			client.removeUser(this, nick, message);
    		}
    }

    public String getServer() {
    		return server;
    }
    
    public Client getClient() {
    		return client;
    }

    public Date getDate() {
    		return date;
    }

    public void addReadMessageListener(Client client) {
    		this.client = client;
    }

    public Channel makeChannel(String name) {
    		Channel channel = client.makeChannel(this, name);
    		addChannel(channel);

    		return channel;
    }
    
    public void addChannel(Channel channel) {
    		channelList.put(channel.toString(), channel);
    }
    
    public void removeChannel(Channel channel) {
    		client.removeChannel(channel);
    		channelList.remove(channel);
    }

    public Channel getChannel(String name) {
    		return (Channel )channelList.get(name);
    }

    public Enumeration getChannels() {
    		return channelList.elements();
    }

    public void readMessage(String string) {
    		StringTokenizer st = new StringTokenizer(string, " ", false);
	
    		String prefix = "";
    		if(string.startsWith(":")) {
    			prefix = st.nextToken();
    		}
	
    		String command = st.nextToken();
    		String params = client.encToUNICODE(st.nextToken(CRLF).trim());
    		st = null;

	// debug
    		if(!command.equals("PING") && DEBUG) {
    			System.out.println(string);
    		}

    		if(command.length() == 3) {
    			commandMap.getCommand("NUMERICREPLY").execute(command, params);
    		} else {
    			if(prefix.equals("")) {
    				commandMap.getCommand(command).execute(params);
    			} else {
    				commandMap.getCommand(command).execute(prefix, params);
    			}
    		}
    }
    
    public final void writeMessage(String message) throws IOException {
    		io.writeMessage(message);
    }
    
    public void directMessage(String message)  throws IOException {
    		writeMessage(message);
    }
    
    public void NICK(String nick)  throws IOException {
    		writeMessage("NICK " + nick);
    }
    
    public void USER(String message)  throws IOException {
    		writeMessage("USER " + message);
    }

    public void PASS(String pass) throws IOException {
    		writeMessage("PASS " + pass);
    		pass = null;
    }
    
    public void PRIVMSG(String channel, String message)  throws IOException {
    		writeMessage("PRIVMSG " + channel + " :" + message);
    }
    
    public void NOTICE(String channel, String message)  throws IOException {
    		writeMessage("NOTICE " + channel + " :" + message);
    }

    public void PONG(String message)  throws IOException {
    		writeMessage("PONG " + message);
    }
    
    public synchronized void JOIN(String message)  throws IOException {
    		if(!connected) {
    			try {
				wait();
			} catch (InterruptedException e) {
				setException(e);
			}
    		}
    		writeMessage("JOIN " + message);
    }

    public void MODE(String message)  throws IOException {
    		writeMessage("MODE " + message);
    }

    protected void protectedMODE(String message) throws IOException {
    		writeMessage("MODE " + client.toENC(message));
    }

    public void PART(String message)  throws IOException {
    		writeMessage("PART " + message);
    }

    public void TOPIC(String channel, String topic)  throws IOException {
    		writeMessage("TOPIC " + client.toENC(channel) + " :" + topic);
    }

    public void QUIT()  throws IOException {
    		writeMessage("QUIT");
    }

    void setException(Exception e) {
    		if(client != null) {
    			client.setException(e);
    		}
    }
}
